@@ -157,7 +157,7 @@ class type request = object
     method meth: meth
     method uri: string
     method path: string
-    method param: ?meth:meth -> string -> string
+    method param: ?meth:meth -> ?default:string -> string -> string
     method paramAll: ?meth:meth -> string -> string list
     method params: (string * string) list
     method params_GET: (string * string) list
